/*
Copyright (c) 2007-2010 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.
* Neither the name of Nokia Corporation nor the names of its contributors
 may be used to endorse or promote products derived from this software
 without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Description:  
*/
 
#ifndef STACK_H
#define STACK_H

/* INCLUDE FILES */
#include <string.h>
#include "CommanHeader.h"


/*
C++ Class that defines Stack
*/
class Stack {
enum 
	{
	EMaxProduction = 1000
	};

public:

	Stack() : iStatckTop(-1) 
	{
	}

	/**
	* Destructor
	* Delete all the items those are not yet consumed by the consumers
	*/
	~Stack()
	{
		ProducedItem* item = NULL; 
		do
			{
			item = Pop();
			delete item;
			} while(item != NULL);
	}

	/**
	* Push
	* New Item is created on Stack for every push operation
	* 
	* @param aItem - Item to be Pushed
	*
	* @return Nothing
	*/
	void Push( ProducedItem* aItem ) 
	{
		//If there is enough space to put item onto stack
		if(iStatckTop < EMaxProduction-1)
			{
			ProducedItem* item = new ProducedItem;
			memcpy(item, aItem, sizeof(ProducedItem));
			iItems[++iStatckTop] = item;
			}
		//else Ignore it!
	}

	/**
	* Pop
	* Item is sent to the Consumer who is doing the Pop. 
	* Ownership will also ne given.
	* So, this item has to be deleted by the Consumer.
	* 
	* @param  Nothing 
	*
	* @return - Item Poped
	*/
	ProducedItem* Pop() 
	{
		if(iStatckTop != -1) 
		{
			ProducedItem* item = iItems[iStatckTop];
			iItems[iStatckTop--] = NULL;
			return item;
		}
		return NULL;
	}

private:
	ProducedItem*	iItems[EMaxProduction];
	int				iStatckTop;
};

#endif /*STACK_H*/

/*  End of File */
